@@ -1,9 +1,11 @@
+import datetime
+
 import pytz
 from gi.repository import GObject, Gdk, Gtk
 
 from abeluna.settings import settings
 from abeluna.sync import Calendar, server
-from abeluna.widgets import DropdownSelectWidget, ErrorDialog
+from abeluna.widgets import DateTimePickerWidget, DropdownSelectWidget, ErrorDialog
 
 
 class CalendarEditor(Gtk.Grid):
@@ -179,6 +181,59 @@ class SettingsWindow(Gtk.Window):
         self.general_page_grid.attach_next_to(self.priority_label, self.autosync_label, Gtk.PositionType.BOTTOM, 2, 1)
         self.general_page_grid.attach_next_to(self.priority_selector, self.priority_label, Gtk.PositionType.RIGHT, 4, 1)
 
+        self.hide_completed_label = Gtk.Label(label='Hide completed tasks')
+        self.hide_completed_selector = Gtk.CheckButton()
+        self.hide_completed_selector.set_active(int(settings.HIDE_COMPLETED))
+        self.general_page_grid.attach_next_to(
+            self.hide_completed_label,
+            self.priority_label,
+            Gtk.PositionType.BOTTOM,
+            2,
+            1,
+        )
+        self.general_page_grid.attach_next_to(
+            self.hide_completed_selector,
+            self.hide_completed_label,
+            Gtk.PositionType.RIGHT,
+            4,
+            1,
+        )
+
+        self.all_day_due_time_label = Gtk.Label('All day tasks due time')
+        self.all_day_due_time_popover = Gtk.Popover()
+        self.all_day_due_time_picker = DateTimePickerWidget(
+            visible_parts='TIME',
+            selected_date=datetime.datetime.strptime(settings.ALL_DAY_DUE_TIME, '%H:%M'),
+        )
+        self.all_day_due_time_popover.add(self.all_day_due_time_picker)
+        self.all_day_due_time_button = Gtk.Button(label=str(self.all_day_due_time_picker))
+
+        def on_clicked_all_day_due_time(obj):
+            self.all_day_due_time_popover.set_relative_to(self.all_day_due_time_button)
+            self.all_day_due_time_popover.show_all()
+            self.all_day_due_time_popover.popup()
+        self.all_day_due_time_button.connect('clicked', on_clicked_all_day_due_time)
+
+        def on_update_all_day_due_time(obj):
+            self.all_day_due_time_popover.popdown()
+            self.all_day_due_time_button.set_label(str(self.all_day_due_time_picker))
+        self.all_day_due_time_picker.connect('updated-date', on_update_all_day_due_time)
+
+        self.general_page_grid.attach_next_to(
+            self.all_day_due_time_label,
+            self.hide_completed_label,
+            Gtk.PositionType.BOTTOM,
+            2,
+            1,
+        )
+        self.general_page_grid.attach_next_to(
+            self.all_day_due_time_button,
+            self.all_day_due_time_label,
+            Gtk.PositionType.RIGHT,
+            4,
+            1,
+        )
+
         self.saved_label = Gtk.Label(label=' ')
         self.saved_label.set_xalign(0.95)
         self.saved_label.set_yalign(0.75)
@@ -191,12 +246,16 @@ class SettingsWindow(Gtk.Window):
             timezone = self.timezone_combo.get_active_id()
             autosync_interval = self.autosync_selector.get_active_id()
             priority = self.priority_selector.get_active_id()
+            hide_completed = str(int(self.hide_completed_selector.get_active()))
+            all_day_due_time = self.all_day_due_time_picker.get_selected_date().strftime('%H:%M')
 
             failed_settings = []
             for obj, name in (
                 (timezone, 'timezone'),
                 (autosync_interval, 'autosync interval'),
                 (priority, 'priority'),
+                (hide_completed, 'hidden completed tasks'),
+                (all_day_due_time, 'all day due time'),
             ):
                 if obj is None:
                     failed_settings.append(name)
@@ -207,14 +266,25 @@ class SettingsWindow(Gtk.Window):
                     'Setting for {} is invalid. Please try again.'.format(', '.join(failed_settings)),
                 ).run_and_wait()
             else:
+                rebuild_todolist = False
                 if settings.TIMEZONE != timezone:
                     settings.TIMEZONE = timezone
-                    self.parent.todolist_window.rebuild_todolist()
+                    rebuild_todolist = True
+                if settings.HIDE_COMPLETED != hide_completed:
+                    settings.HIDE_COMPLETED = hide_completed
+                    rebuild_todolist = True
+                if settings.ALL_DAY_DUE_TIME != all_day_due_time:
+                    settings.ALL_DAY_DUE_TIME = all_day_due_time
+                    rebuild_todolist = True
                 if settings.AUTOSYNC_INTERVAL != autosync_interval:
                     settings.AUTOSYNC_INTERVAL = autosync_interval
                     server.restart_autosync_thread()
                 settings.PRIORITIZE_ON_CONFLICT = priority
                 settings.commit()
+
+                if rebuild_todolist:
+                    self.parent.todolist_window.rebuild_todolist()
+
                 self.saved_label.set_label('Saved!')
                 GObject.timeout_add_seconds(5, clear_saved_label)
 
@@ -222,7 +292,7 @@ class SettingsWindow(Gtk.Window):
         self.save_button.set_margin_top(20)
         self.save_button.connect('clicked', save_button_clicked)
 
-        self.general_page_grid.attach(self.save_button, 4, 4, 2, 1)
+        self.general_page_grid.attach(self.save_button, 4, 6, 2, 1)
         self.general_page_grid.attach_next_to(self.saved_label, self.save_button, Gtk.PositionType.LEFT, 2, 1)
 
         padding = Gtk.Box()
@@ -332,27 +402,41 @@ class SettingsWindow(Gtk.Window):
         self.scrollable_server_view.set_propagate_natural_width(True)
         self.scrollable_server_view.set_shadow_type(type=Gtk.ShadowType.ETCHED_OUT)
         self.scrollable_server_view.add(self.server_view)
-        self.servers_page_grid.attach(self.scrollable_server_view, 0, 0, 6, 6)
+        self.servers_page_grid.attach(self.scrollable_server_view, 0, 0, 8, 6)
 
         self.server_add_button = Gtk.Button(label='Add')
+        self.server_clone_button = Gtk.Button(label='Clone')
         self.server_edit_button = Gtk.Button(label='Modify')
         self.server_delete_button = Gtk.Button(label='Delete')
 
+        self.server_clone_button.set_sensitive(False)
         self.server_edit_button.set_sensitive(False)
         self.server_delete_button.set_sensitive(False)
 
         def on_todo_tree_selection_changed(todo_tree_selection):
             current_it = todo_tree_selection.get_selected()[1]
             if current_it is None:
+                self.server_clone_button.set_sensitive(False)
                 self.server_edit_button.set_sensitive(False)
                 self.server_delete_button.set_sensitive(False)
             else:
+                self.server_clone_button.set_sensitive(True)
                 self.server_edit_button.set_sensitive(True)
                 self.server_delete_button.set_sensitive(True)
         self.server_view.get_selection().connect('changed', on_todo_tree_selection_changed)
 
-        def on_server_add(button):
-            self.server_editor.set_data({})
+        def on_server_create(button, clone_selected):
+            if clone_selected:
+                it = self.server_view.get_selection().get_selected()[1]
+                if it is None:
+                    return
+                uid = self.server_todo_store[it][3]
+                data = settings.CALENDARS[uid].copy()
+                data.pop('uid')
+            else:
+                data = {}
+
+            self.server_editor.set_data(data)
             rect = Gdk.Rectangle()
             rect.width = button.get_allocation().width
             rect.height = button.get_allocation().height
@@ -362,7 +446,8 @@ class SettingsWindow(Gtk.Window):
             self.server_editor_popover.set_position(Gtk.PositionType.TOP)
             self.server_editor_popover.show_all()
             self.server_editor_popover.popup()
-        self.server_add_button.connect('clicked', on_server_add)
+        self.server_add_button.connect('clicked', on_server_create, False)
+        self.server_clone_button.connect('clicked', on_server_create, True)
 
         def on_server_edit(button):
             it = self.server_view.get_selection().get_selected()[1]
@@ -387,7 +472,10 @@ class SettingsWindow(Gtk.Window):
             self.server_add_button, self.scrollable_server_view, Gtk.PositionType.BOTTOM, 2, 1,
         )
         self.servers_page_grid.attach_next_to(
-            self.server_edit_button, self.server_add_button, Gtk.PositionType.RIGHT, 2, 1,
+            self.server_clone_button, self.server_add_button, Gtk.PositionType.RIGHT, 2, 1,
+        )
+        self.servers_page_grid.attach_next_to(
+            self.server_edit_button, self.server_clone_button, Gtk.PositionType.RIGHT, 2, 1,
         )
         self.servers_page_grid.attach_next_to(
             self.server_delete_button, self.server_edit_button, Gtk.PositionType.RIGHT, 2, 1,
